#!/usr/bin/env python

from email import message_from_file
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from smtplib import SMTP
from imiptools.content import handle_itip_part
import imip_resource
import sys

MESSAGE_SENDER = "resources+agent@example.com"

MESSAGE_SUBJECT = "Calendar system message"

MESSAGE_TEXT = """\
This is a response to a calendar message sent by your calendar program.
"""

# Postfix exit codes.

EX_TEMPFAIL     = 75

# Permitted iTIP content types.

itip_content_types = [
    "text/calendar",                        # from RFC 6047
    "text/x-vcalendar", "application/ics",  # other possibilities
    ]

# Sending of outgoing messages.

def sendmail(sender, recipients, data):
    smtp = SMTP("localhost")
    smtp.sendmail(sender, recipients, data)
    smtp.quit()

# Processing of incoming messages.

def process(f, original_recipients, recipients, this_sender):

    """
    Process content from the stream 'f' accompanied by the given
    'original_recipients' and 'recipients', together with details of
    'this_sender' (the sender address on messages sent by this service).
    """

    msg = message_from_file(f)
    senders = msg.get_all("Reply-To") or msg.get_all("From")
    original_recipients = original_recipients or get_all_values(msg, "To")

    # Handle messages with iTIP parts.

    all_parts = []

    for part in msg.walk():
        if part.get_content_type() in itip_content_types and \
            part.get_param("method"):

            # NOTE: Act on behalf of resources for now.

            all_parts += handle_itip_part(part, original_recipients, imip_resource.handlers)

    # Pack the parts into a single message.

    if all_parts:
        text_part = MIMEText(MESSAGE_TEXT)
        all_parts.insert(0, text_part)
        message = MIMEMultipart("alternative", _subparts=all_parts)
        message.preamble = MESSAGE_TEXT

        message["From"] = this_sender
        for sender in senders:
            message["To"] = sender
        message["Subject"] = MESSAGE_SUBJECT

        if "-d" in sys.argv:
            print message
        else:
            sendmail(this_sender, senders, message.as_string())

def get_all_values(msg, key):
    l = []
    for v in msg.get_all(key) or []:
        l += [s.strip() for s in v.split(",")]
    return l

def main():

    "Interpret program arguments and process input."

    # Obtain the different kinds of recipients plus sender address.

    original_recipients = []
    recipients = []
    senders = []

    l = []

    for arg in sys.argv[1:]:

        # Switch to collecting recipients.

        if arg == "-o":
            l = original_recipients
        elif arg == "-r":
            l = recipients

        # Switch to collecting senders.

        elif arg == "-s":
            l = senders

        # Ignore debugging options.

        elif arg == "-d":
            pass
        else:
            l.append(arg)

    process(sys.stdin, original_recipients, recipients, senders and senders[0] or MESSAGE_SENDER)

if __name__ == "__main__":
    if "-d" in sys.argv[1:]:
        main()
    else:
        try:
            main()
        except SystemExit, value:
            sys.exit(value)
        except Exception, exc:
            type, value, tb = sys.exc_info()
            print >>sys.stderr, "Exception %s at %d" % (exc, tb.tb_lineno)
            sys.exit(EX_TEMPFAIL)
    sys.exit(0)

# vim: tabstop=4 expandtab shiftwidth=4
